using System;
using System.ComponentModel;
using System.Windows.Forms;

using Smdn.Collections;
using Smdn.Formats;
using Smdn.Formats.Thbgm;
using Smdn.Windows.Forms;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public class ExtendedTrackListView : TrackListView<ExtendedTrackListViewItem> {
    public event EventHandler TrackEffectPropertyChanged;

    public ExtendedTrackListView()
    {
      Columns.Add(new TrackListViewColumnHeader("OutputFileLength", ThbgmExtractor.GetText("OutputFileLength"), 75));

      extractProgressColumnHeader = new TrackListViewColumnHeader("ExtractProgress", ThbgmExtractor.GetText("ExtractionProgress"), 200);
      lastExtractProgressColumnWidth = extractProgressColumnHeader.Width;
    }

    public void ShowExtractProgressColumn()
    {
      if (Columns.Contains(extractProgressColumnHeader))
        Columns.Remove(extractProgressColumnHeader);

      Columns.Insert(2, extractProgressColumnHeader);

      extractProgressColumnHeader.Width = lastExtractProgressColumnWidth;

      UpdateItems();
      UpdateAllTracksItemSubItems();
    }

    public void HideExtractProgressColumn()
    {
      if (Columns.Contains(extractProgressColumnHeader)) {
        lastExtractProgressColumnWidth = extractProgressColumnHeader.Width;

        Columns.Remove(extractProgressColumnHeader);
      }

      foreach (var item in TrackItems) {
        item.ExtractProgress = null;
      }

      UpdateItems();
      UpdateAllTracksItemSubItems();
    }

    protected override string GetAllTracksSubItemText(TrackListViewColumnHeader header)
    {
      switch (header.BindingValueName) {
        case "Title":
          if (Product == null)
            return string.Empty;
          else
            return Product.Title;

        case "ExtractProgress":
          if (extractProgress == null)
            return string.Empty;
          else
            // TODO: translation
            return string.Format("Undone {0} Success {1} Failed {2}",
                                 extractProgress.TotalCount - (extractProgress.SucceededCount + extractProgress.FailedCount),
                                 extractProgress.SucceededCount,
                                 extractProgress.FailedCount);

        case "OutputFileLength": {
          var totalLength = 0L;

          foreach (var item in TrackItems) {
            totalLength += item.Track.TrackInfo.StreamFormat.ToAlignedByteCount(item.Track.Effect.TotalLength);
          }

          return string.Format(SIPrefixFormat.CurrentInfo, "{0:f}", totalLength);
        }

        default:
          return base.GetAllTracksSubItemText(header);
      }
    }

    private void UpdateItems()
    {
      foreach (var item in TrackItems) {
        item.InternalUpdateSubItems();
      }
    }

    public void UpdateExtractProgress(ExtractBackgroundWorker.IExtractTrackProgress extractProgress)
    {
      var item = TrackItems.Find(delegate(ExtendedTrackListViewItem i) {
        return (i.Track.TrackInfo == extractProgress.Track.TrackInfo);
      });

      if (item == null)
        return;

      item.ExtractProgress = extractProgress;
      item.InternalUpdateSubItems();
    }

    public void UpdateExtractProgress(ExtractBackgroundWorker.IExtractProgress extractProgress)
    {
      this.extractProgress = extractProgress;

      UpdateAllTracksItemSubItems();
    }

    internal protected virtual void OnTrackEffectPropertyChanged(EventArgs e)
    {
      var ev = this.TrackEffectPropertyChanged;

      if (ev != null)
        ev(this, e);
    }

    protected override ListViewSubItemEditorBase CreateSubItemEditor(TrackListViewItem item, TrackListViewColumnHeader header, ListViewItem.ListViewSubItem subItem)
    {
      switch (header.BindingValueName) {
        case "OutputFileLength":
          return null;

        case "ExtractProgress":
          return null;

        default:
          return base.CreateSubItemEditor(item, header, subItem);
      }
    }

    private TrackListViewColumnHeader extractProgressColumnHeader;
    private int lastExtractProgressColumnWidth;
    private ExtractBackgroundWorker.IExtractProgress extractProgress;
  }
}
