using System;
using System.ComponentModel;
using System.Windows.Forms;

using Smdn.Formats;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public class ExtendedTrackListViewItem : TrackListViewItem {
    internal ExtractBackgroundWorker.IExtractTrackProgress ExtractProgress {
      get; set;
    }

    public ExtendedTrackListViewItem()
    {
    }

    internal void InternalUpdateSubItems()
    {
      UpdateSubItems();
    }

    protected override void OnEffectPropertyChanged(PropertyChangedEventArgs e)
    {
      var listView = ListView as ExtendedTrackListView;

      if (listView != null)
        listView.OnTrackEffectPropertyChanged(e);

      base.OnEffectPropertyChanged(e);
    }

    protected override string GetSubItemText(TrackListViewColumnHeader header)
    {
      switch (header.BindingValueName) {
        case "OutputFileLength":
          return string.Format(SIPrefixFormat.CurrentInfo,
                               "{0:f}",
                               Track.TrackInfo.StreamFormat.ToAlignedByteCount(Track.Effect.TotalLength));

        case "ExtractProgress":
          // TODO: graphical progress view
          if (ExtractProgress == null)
            return string.Empty;
          else
            return string.Format("{0:P1} {1}", ExtractProgress.ProgressPercent, ExtractProgress.StatusText);

        default:
          return base.GetSubItemText(header);
      }
    }
  }
}
