using System;
using System.Collections.Generic;
using System.IO;

using Smdn.Collections;
using Smdn.Formats.Playlists;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public static class TrackPlaylist {
    public static void Save(string pathPlaylist, IDictionary<Track, string> trackAndPaths, PlaylistFormat format)
    {
      var playlistBaseDirectory = Path.GetDirectoryName(pathPlaylist);
      var product = trackAndPaths.Keys.First().TrackInfo.Product;

      var playlist = new Playlist(trackAndPaths.ConvertAll(delegate(KeyValuePair<Track, string> pair) {
        Uri trackLocation;

        if (Path.GetDirectoryName(pair.Value).StartsWith(playlistBaseDirectory))
          // set relative path
          //trackLocation = new Uri(new Uri(string.Format("file://{0}", playlistBaseDirectory)), "./" + pair.Value.Replace(playlistBaseDirectory, string.Empty));
          trackLocation = new Uri("." + pair.Value.Replace(playlistBaseDirectory, string.Empty), UriKind.Relative);
        else
          // set absolute path
          trackLocation = new Uri(string.Format("file://{0}",pair.Value));

        var entry = new PlaylistEntry(trackLocation);
        var track = pair.Key;

        entry.Title = track.TrackInfo.Title;
        entry.TrackNumber = track.TrackInfo.TrackNumber;
        entry.AlbumTitle = track.TrackInfo.Product.Title;
        entry.Duration = track.Effect.TotalLength;

        return entry;
      }));

      playlist.Creator = product.Creator;
      playlist.CreationDate = DateTimeOffset.Now;
      playlist.Generator = ThbgmExtractor.VersionString;
      playlist.Title = product.Title;

      playlist.Save(pathPlaylist, format);
    }
  }
}
