using System;
using System.ComponentModel;
using System.Windows.Forms;

namespace Smdn.Applications.ThbgmExtractor {
  public class TrackTagListViewItem : ListViewItem {
    public TrackTag TrackTag {
      get { return trackTag; }
      set
      {
        if (value == null)
          throw new ArgumentNullException();

        if (trackTag != null)
          trackTag.PropertyChanged -= TrackTag_PropertyChanged;

        trackTag = value;
        trackTag.PropertyChanged += TrackTag_PropertyChanged;

        UpdateSubItems();
      }
    }

    protected internal virtual void UpdateSubItems()
    {
      if (SubItems.Count != ListView.Columns.Count) {
        SubItems.Clear();

        for (var i = 0; i < ListView.Columns.Count; i++) {
          SubItems.Add(string.Empty);
        }
      }

      for (var i = 0; i < ListView.Columns.Count; i++) {
        SubItems[i].Text = GetSubItemText(ListView.Columns[i] as TrackTagListViewColumnHeader);
      }
    }

    protected virtual string GetSubItemText(TrackTagListViewColumnHeader header)
    {
      switch (header.BindingValueName) {
        case "TrackNumber":             return TrackTag.TrackInfo.TrackNumber.ToString();
        case "IsDirty":                 return TrackTag.IsDirty.ToString();

        case "Title":           return TrackTag.Title;
        case "OutputFilename":  return TrackTag.FormattedOutputFileName;
        case "Album":           return TrackTag.Album;
        case "Performers":      return TrackTag.JoinedPerformers;
        case "AlbumArtists":    return TrackTag.JoinedAlbumArtists;
        case "Composers":       return TrackTag.JoinedComposers;
        case "Genres":          return TrackTag.JoinedGenres;
        case "Comment":         return TrackTag.Comment;
        default: return string.Empty;
      }
    }

    private void TrackTag_PropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      if (sender == trackTag)
        OnTrackTagPropertyChanged(e);
    }

    protected virtual void OnTrackTagPropertyChanged(PropertyChangedEventArgs e)
    {
      UpdateSubItems();
    }

    private TrackTag trackTag;
  }
}
